'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _sourceMapSupport2 = require('source-map-support');

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _http = require('http');

var _http2 = _interopRequireDefault(_http);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _https = require('https');

var _https2 = _interopRequireDefault(_https);

var _request = require('request');

var _request2 = _interopRequireDefault(_request);

var _wdioLogger = require('wdio-logger');

var _wdioLogger2 = _interopRequireDefault(_wdioLogger);

var _events = require('events');

var _events2 = _interopRequireDefault(_events);

var _utils = require('./utils');

var _package = require('../package.json');

var _package2 = _interopRequireDefault(_package);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

(0, _sourceMapSupport2.install)();


const log = (0, _wdioLogger2.default)('webdriver');
const agents = {
    http: new _http2.default.Agent({ keepAlive: true }),
    https: new _https2.default.Agent({ keepAlive: true })
};

class WebDriverRequest extends _events2.default {
    constructor(method, endpoint, body) {
        super();
        this.method = method;
        this.endpoint = endpoint;
        this.defaultOptions = {
            method,
            body,
            followAllRedirects: true,
            json: true,
            headers: {
                'Connection': 'keep-alive',
                'Accept': 'application/json',
                'User-Agent': 'webdriver/' + _package2.default.version
            }
        };
    }

    makeRequest(options, sessionId) {
        const fullRequestOptions = Object.assign(this.defaultOptions, this._createOptions(options, sessionId));
        this.emit('request', fullRequestOptions);
        return this._request(fullRequestOptions, options.connectionRetryCount);
    }

    _createOptions(options, sessionId) {
        const requestOptions = {
            agent: agents[options.protocol]

            /**
             * if we don't have a session id we set it here, unless we call commands that don't require session ids, for
             * example /sessions. The call to /sessions is not connected to a session itself and it therefore doesn't
             * require it
             */
        };if (this.endpoint.match(/:sessionId/) && !sessionId) {
            throw new Error('A sessionId is required for this command');
        }

        requestOptions.uri = _url2.default.parse(`${options.protocol}://` + `${options.hostname}:${options.port}` + _path2.default.resolve(`${options.path}${this.endpoint.replace(':sessionId', sessionId)}`));

        /**
         * Check for custom authorization header
         */
        if (typeof options.headers === 'object') {
            requestOptions.headers = Object.assign(requestOptions.headers, options.headers);
        }

        /**
         * Add query parameters to request options if it is an object
         */
        if (typeof this.defaultOptions.queryParams === 'object') {
            requestOptions.qs = options.queryParams;
        }

        /**
         * send authentication credentials only when creating new session
         */
        if (this.endpoint === '/session' && options.user && options.key) {
            requestOptions.auth = {
                user: options.user,
                pass: options.key
            };
        }

        return requestOptions;
    }

    _request(fullRequestOptions, totalRetryCount = 0, retryCount = 0) {
        log.info(`[${fullRequestOptions.method}] ${fullRequestOptions.uri.href}`);

        if (fullRequestOptions.body && Object.keys(fullRequestOptions.body).length) {
            log.info('DATA', fullRequestOptions.body);
        }

        return new Promise((resolve, reject) => (0, _request2.default)(fullRequestOptions, (err, response, body) => {
            const error = new Error(err || (body.value ? body.value.message : body));

            /**
             * Resolve only if successful response
             */
            if (!err && (0, _utils.isSuccessfulResponse)(response)) {
                this.emit('response', { result: body });
                return resolve(body);
            }

            if (retryCount >= totalRetryCount) {
                log.error('Request failed after retry due to', error);
                this.emit('response', { error });
                return reject(error);
            }

            ++retryCount;
            this.emit('retry', { error, retryCount });
            log.warn('Request failed due to', error.message);
            log.info(`Retrying ${retryCount}/${totalRetryCount}`);
            this._request(fullRequestOptions, totalRetryCount, retryCount).then(resolve, reject);
        }));
    }
}
exports.default = WebDriverRequest;
module.exports = exports['default'];